/******************************************************************************
* Copyright 2013-2014 - Alexander 'Wareya' Nadeau <wareya@gmail.com>
* 
* This software is provided 'as-is', without any express or implied warranty.
* In no event will the authors be held liable for any damages arising from the
* use of this software.
* 
* Permission is granted to anyone to use this software and/or its source code
* for any purpose, including commercial applications, and to alter it and
* distribute it freely, subject to the following conditions:
*
*    1. The origin of the software must not be misrepresented.
*
*    2. Modified versions of the software or its source code must not
*    be misrepresented as being the original software or source code.
*
*    3. This notice may not be removed from, or altered in, any source
*    distribution of the software.
*
*        3a. This notice is not required to be included in any compiled
*        distribution of the software, unless said distribution also
*        constitutes a source distribution.
*        
*    4. In case of use or inclusion of this software in a commercial
*    product, a proper acknowledgment of this software inside the final
*    product is required.
******************************************************************************/

global.serverPluginsRequired = 1;
globalvar WarPacksPPS;
WarPacksPPS = argument1;

globalvar PickupController;
PickupController = object_add();
object_set_persistent(PickupController,true);

if(global.isHost)
{
    object_event_add(PickupController, ev_other, ev_room_start, '
        ds_list_clear(pickups);
        buffer_clear(joinerbuffer);
        
        desire = global.currentMap + ".ini";
        // Does it exist, and can we access it?
        var b2;
        if(file_exists("Maps/" + desire) and !file_exists(desire))
        {
            // We have to copy it to the program directory to load it because GAME MAKER
            file_copy("Maps/" + desire, desire);
            ini_open(desire);
            var numpacks;
            numpacks = 0;
            // get inner buffer ready so that its easier to write to it
            b2 = buffer_create();
            // find each health pack in the ini, stop when we hit an invalid one
            while (ini_key_exists("HealthPackPlugin", "HealthPack" + string(numpacks)))
            {
                var itsx;
                var itsy;
                
                coords = ini_read_string("HealthPackPlugin", "HealthPack" + string(numpacks), "");
                
                coordslist = split(coords, " ", 2);
                itsx = real(ds_list_find_value(coordslist, 0));
                itsy = real(ds_list_find_value(coordslist, 1));
                ds_list_destroy(coordslist);
                
                var t;
                t = instance_create(itsx, itsy, PickupSpot);
                ds_list_add(pickups, t);
                
                write_ushort(b2, itsx);
                write_ushort(b2, itsy);
                
                numpacks += 1;
            }
            ini_close();
            file_delete(desire);
            
            // work on send buffer
            write_ubyte(joinerbuffer, 73); // magic
            write_ubyte(joinerbuffer, ds_list_size(pickups)); // count
            write_buffer(joinerbuffer,b2); // list of coords
            
            buffer_destroy(b2);
            
            with(Player)
            {
                PluginPacketSendTo(WarPacksPPS, PickupController.joinerbuffer, id);
                HealthPackServiced = true;
            }
        }
    ');
    object_event_add(Player, ev_step, ev_step_end, '
        // needs to be here because socket hasnt been set in ev_create yet
        if(!variable_local_exists("HealthPackServiced") and id != global.myself)
        {
            if(instance_exists(PickupController)) // sanity check
            {
                if(ds_list_size(PickupController.pickups) > 0) // have generated healthpacks
                {
                    if(!variable_local_exists("HealthPackServiced"))
                        PluginPacketSendTo(WarPacksPPS, PickupController.joinerbuffer, id);
                    
                    var f;
                    f = buffer_create();
                    write_ubyte(f, 78);
                    var count;
                    for(count = 0; count < ds_list_size(PickupController.pickups); count += 1)
                    {
                        t = ds_list_find_value(PickupController.pickups, count);
                        write_ubyte(f, t.progress/2);
                    }
                    PluginPacketSendTo(WarPacksPPS, f, id);
                    buffer_destroy(f);
                }
            }
            HealthPackServiced = true;
        }
    ');
    object_event_add(PickupController, ev_draw, 0, '
        if ( instance_exists(Player) or instance_exists(Spectator) ) // ingame
        {
            // draw coordinates for pickup placement
            draw_set_halign(fa_left);
            draw_set_valign(fa_top);
            draw_set_color(c_white);
            draw_set_alpha(1);
            if(global.myself)
                if(global.myself.object > 0)
                    draw_text(view_xview+10, view_yview+10, string(round(global.myself.object.x)) + " " + string(round(global.myself.object.bbox_bottom)-16));
        }
    ');
}
if(!global.isHost)
{
    object_event_add(PickupController, ev_step, ev_step_end, '
        if ( instance_exists(Player) or instance_exists(Spectator) ) // ingame
        {
            while (PluginPacketGetBuffer(WarPacksPPS) != -1)
            {
                d = PluginPacketGetBuffer(WarPacksPPS);
                var what;
                what = read_ubyte(d);
                if(what == 73)
                {
                    ds_list_clear(pickups);
                    var count;
                    numpacks = read_ubyte(d);
                    for(count = 0; count < numpacks; count += 1)
                    {
                        var px;
                        var py;
                        var t;
                        px = read_ushort(d);
                        py = read_ushort(d);
                        t = instance_create(px, py, PickupSpot);
                        ds_list_add(pickups, t);
                    }
                }
                if(what == 78)
                {
                    var count;
                    var t;
                    for(count = 0; count < ds_list_size(pickups); count += 1)
                    {
                        t = ds_list_find_value(pickups, count);
                        t.progress = read_ubyte(d)*2;
                    }
                }
                if(what == 24)
                {
                    var which;
                    var who;
                    
                    which = ds_list_find_value(pickups, read_ubyte(d));
                    who = ds_list_find_value(global.players, read_ubyte(d));
                    
                    with(which)
                    {
                        if(instance_exists(who.object))
                        {
                            if (stage >= 1)
                            {
                                who.object.hp += 25;
                            }
                            if (stage == 2)
                            {
                                who.object.hp += round((who.object.maxHp * 0.1)/5)*5;
                            }
                        }
                        stage = 0;
                        progress = 0;
                    }
                }
                PluginPacketPop(WarPacksPPS);
            }
        }
    ');
}

var FUCK;
FUCK = instance_create(0,0,PickupController);
FUCK.joinerbuffer = buffer_create();
FUCK.pickups = ds_list_create();

globalvar PickupSpot;
PickupSpot = object_add();
globalvar Pickup1S;
Pickup1S = sprite_add(directory + "/healthsmall.png", 2, 1, 0, 15, 15);
globalvar Pickup2S;
Pickup2S = sprite_add(directory + "/healthlarge.png", 2, 1, 0, 15, 15);
globalvar PickupSpotS;
PickupSpotS = sprite_add(directory + "/healthpulse.png", 6, 1, 0, 15, 15);
sprite_collision_mask(PickupSpotS,false,2,8,8,24,24,1,0);

object_event_add(PickupSpot, ev_create, 0, '
    // We dont use this variable for rendering, but it gives us the right collision mask
    sprite_index = PickupSpotS;
    // Bases animation rate
    image_speed = global.delta_factor/30*6;
    // Which level of healthpack
    stage = 0;
    if(!variable_local_exists("progress")) // fucking code execution order in this BULLSHIT
    {
        // Recharging progress
        progress = 0;
    }
');

if(global.isHost)
{
    object_event_add(PickupSpot, ev_collision, Character, '
        if(other.hp >= other.maxHp or stage < 1)
            exit;
        if (stage >= 1)
        {
            other.hp += 25;
        }
        if (stage == 2)
        {
            other.hp += round((other.maxHp * 0.1)/5)*5;
        }
        stage = 0;
        progress = 0;
        
        var who;
        who = ds_list_find_index(global.players, other.player);
        
        var buf;
        buf = buffer_create();
        write_ubyte(buf, 24);
        write_ubyte(buf, ds_list_find_index(PickupController.pickups, id));
        write_ubyte(buf, who);
        with (Player)
        {
            if (id != global.myself)
                PluginPacketSendTo(WarPacksPPS, buf, id)
        }
        buffer_destroy(buf);
    ');
}
object_event_add(PickupSpot, ev_step, ev_step_end, '
    if (progress >= 30 * 8)
        stage = 1;
    if (progress >= 30 * 14)
        stage = 2;
    if (progress < 30 * 14)
        progress += 1 * global.delta_factor;
');
object_event_add(PickupSpot, ev_draw, 0, '
    
    // draw pickup "stand"
    draw_sprite(PickupSpotS, image_index, x, y);
    // draw pickup if its "spawned"
    if(stage == 1)
        draw_sprite(Pickup1S, image_index, x, y);
    if(stage == 2)
        draw_sprite(Pickup2S, image_index, x, y);
');
